#!/usr/bin/perl -w
#
# $Id: mailman_subs.t,v 1.1 2004/06/20 19:23:08 rich Exp $
# Tests for Email::List::Subscription::Mailman
#

use strict;
use Test::Simple tests => 21;
use Data::Dumper;
use Email::Simple;
use Email::List::Subscription::Mailman;

sub show ($);

# Test 1: Status message. Check we parse out and return
# the expected subscriptions, when the list and password
# are on the same line.
my $message = Email::Simple->new('');

ok(defined($message), 'Created message');

ok($message->header_set('Subject',
			'gnu.org mailing list memberships reminder'),
   'Set Subject');

ok($message->header_set('From', 'mailman-owner@gnu.org'),
   'Set From');

ok($message->header_set('To', 'bob@bogus.com'),
   'Set To');

ok($message->header_set('Date', 'Date: Tue, 01 Jun 2004 05:00:09 -0700'),
   'Set Date');

ok($message->header_set('X-Mailman-Version', '2.1.2'),
   'Set X-Mailman-Version');

my $body = <<EOT;
This is a reminder, sent out once a month, about your gnu.org mailing
list memberships.  It includes your subscription info and how to use
it to change it or unsubscribe from a list.

You can visit the URLs to change your membership status or
configuration, including unsubscribing, setting digest-style delivery
or disabling delivery altogether (e.g., for a vacation), and so on.

In addition to the URL interfaces, you can also use email to make such
changes.  For more info, send a message to the '-request' address of
the list (for example, mailman-request\@gnu.org) containing just the
word 'help' in the message body, and an email message will be sent to
you with instructions.

If you have questions, problems, comments, etc, send them to
mailman-owner\@gnu.org.  Thanks!

Passwords for bob\@bogus.com:

List                                     Password // URL
----                                     --------  
autoconf\@gnu.org                         password
http://mail.gnu.org/mailman/options/autoconf/bob\%40bogus.com

autoconf-patches\@gnu.org                 password
http://mail.gnu.org/mailman/options/autoconf-patches/bob\%40bogus.com

EOT

ok($message->body_set($body), 'Set the body');

my @lines = split(/\n/, $message->body());
my @subscriptions = Email::List::Subscription::Mailman::match($message,
							      \@lines);

print '# '.$#subscriptions."\n";
ok($#subscriptions == 1, 'Expected number of subscriptions found');

my $ok;
my $str;

show($subscriptions[0]);

$ok
    = ($subscriptions[0]->{listname} =~ m/^autoconf\@gnu.org$/i)
    && ($subscriptions[0]->{subscriber} =~ m/^bob\@bogus.com$/i)
    && ($subscriptions[0]->{password} eq 'password')
    && ($subscriptions[0]->{url} =~ m!http://mail.gnu.org/mailman/options/autoconf/bob\%40bogus.com!);

ok($ok, 'First subscription matched');

show($subscriptions[1]);

$ok
    = ($subscriptions[1]->{listname} =~ m/^autoconf-patches\@gnu.org$/i)
    && ($subscriptions[1]->{subscriber} =~ m/^bob\@bogus.com$/i)
    && ($subscriptions[1]->{password} eq 'password')
    && ($subscriptions[1]->{url} =~ m!http://mail.gnu.org/mailman/options/autoconf-patches/bob\%40bogus.com!);

ok($ok, 'Second subscription matched');

# Test 2: Status message. Check we parse out and return
# the expected subscriptions, when we have some messages with the password
# wrapped onto the next line. This happened with some messages
# from Sourceforge for me.
$message = Email::Simple->new('');

ok(defined($message), 'Created message');

ok($message->header_set('Subject',
			'gnu.org mailing list memberships reminder'),
   'Set Subject');

ok($message->header_set('From', 'mailman-owner@gnu.org'),
   'Set From');

ok($message->header_set('To', 'bob@bogus.com'),
   'Set To');

ok($message->header_set('Date', 'Date: Tue, 01 Jun 2004 05:00:09 -0700'),
   'Set Date');

ok($message->header_set('X-Mailman-Version', '2.1.2'),
   'Set X-Mailman-Version');

$body = <<EOT;
This is a reminder, sent out once a month, about your gnu.org mailing
list memberships.  It includes your subscription info and how to use
it to change it or unsubscribe from a list.

You can visit the URLs to change your membership status or
configuration, including unsubscribing, setting digest-style delivery
or disabling delivery altogether (e.g., for a vacation), and so on.

In addition to the URL interfaces, you can also use email to make such
changes.  For more info, send a message to the '-request' address of
the list (for example, mailman-request\@gnu.org) containing just the
word 'help' in the message body, and an email message will be sent to
you with instructions.

If you have questions, problems, comments, etc, send them to
mailman-owner\@gnu.org.  Thanks!

Passwords for bob\@bogus.com:

List                                     Password // URL
----                                     --------  
autoconf\@gnu.org                         password
http://mail.gnu.org/mailman/options/autoconf/bob\%40bogus.com

autoconf-patches\@gnu.org
                 password
http://mail.gnu.org/mailman/options/autoconf-patches/bob\%40bogus.com

libtool\@gnu.org                          password
http://mail.gnu.org/mailman/options/libtool/bob\%40bogus.com

EOT

ok($message->body_set($body), 'Set the body');

@lines = split(/\n/, $message->body());
@subscriptions = Email::List::Subscription::Mailman::match($message,
                                                           \@lines);

print '# '.$#subscriptions."\n";
ok($#subscriptions == 2, 'Expected number of subscriptions found');

show($subscriptions[0]);

$ok
    = ($subscriptions[0]->{listname} =~ m/^autoconf\@gnu.org$/i)
    && ($subscriptions[0]->{subscriber} =~ m/^bob\@bogus.com$/i)
    && ($subscriptions[0]->{password} eq 'password')
    && ($subscriptions[0]->{url} =~ m!http://mail.gnu.org/mailman/options/autoconf/bob\%40bogus.com!);

ok($ok, 'First subscription matched');

show($subscriptions[1]);

$ok
    = ($subscriptions[1]->{listname} =~ m/^autoconf-patches\@gnu.org$/i)
    && ($subscriptions[1]->{subscriber} =~ m/^bob\@bogus.com$/i)
    && ($subscriptions[1]->{password} eq 'password')
    && ($subscriptions[1]->{url} =~ m!http://mail.gnu.org/mailman/options/autoconf-patches/bob\%40bogus.com!);

ok($ok, 'Second subscription matched');

show($subscriptions[2]);

$ok
    = ($subscriptions[2]->{listname} =~ m/^libtool\@gnu.org$/i)
    && ($subscriptions[2]->{subscriber} =~ m/^bob\@bogus.com$/i)
    && ($subscriptions[2]->{password} eq 'password')
    && ($subscriptions[2]->{url} =~ m!http://mail.gnu.org/mailman/options/libtool/bob\%40bogus.com!);

ok($ok, 'Third subscription matched');

#
# show
#

sub show ($)
{
    my $str = Dumper(shift(@_));
    $str =~ s/^/\# /mg;
    print $str;
}
